package example.energy.coolemall;


import schedframe.resources.StandardResourceType;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Processor;
import schedframe.resources.computing.coolemall.ComputeBox1;
import schedframe.resources.computing.coolemall.NodeGroup;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.airthroughput.UserAirThroughputStateName;
import schedframe.resources.devices.Device;
import schedframe.resources.devices.PhysicalResource;
import schedframe.scheduling.manager.tasks.JobRegistry;
import simulator.ConfigurationOptions;
import test.Node_Fan_Mapping;
import example.energy.BaseEnergyEstimationPlugin;

public class CB2EnergyEstimationPlugin extends BaseEnergyEstimationPlugin{

	
	public double estimatePowerConsumption(EnergyEvent event, JobRegistry jobRegistry,
			PhysicalResource resource) {
		
		
		double Pdc;
		
		double Pload_dc = 0;
		double Pothers = 0;
		double Pfans_dc = 0;
		double Pcooling_device = 0;

		ComputingResource room = (ComputingResource) resource;
		
		/**************** IT PART ****************/
		
		/*********** Pload_dc ***********/
		for(ComputingResource cr: room.getChildren()){
			ComputeBox1 cb1 = (ComputeBox1)cr;
			Pload_dc = Pload_dc + cb1.getPowerInterface().getRecentPowerUsage().getValue();
		}
		
		
		
		/**************** NON- IT PART ****************/
		
		/*********** Pothers ***********/		
		double a;//experiment
		try{
			a = ConfigurationOptions.alpha;
		} catch (Exception e){
			a = 0.02;
		}
		
		//if not defined assume alpha = 0.02
		if(a == -1)
			a = 0.02;
		
		Pothers = a * Pload_dc; 
		
		
		
		/************* COOLING PART *************/
		
		Device coolingDevice = null;
		for(Device device: room.getResourceCharacteristic().getDevices()){
			if(device.getType().equals(StandardResourceType.CoolingDevice)){
				coolingDevice = device;
				break;
			}
		}

		
		/*********** Pfans_dc ***********/
		double ro = 1.168;//constant
		double nf;//DEBB
		try{
			nf = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("CRAHFanEfficiency").get(0).getContent()).doubleValue();
		} catch (Exception e){
			nf = 0.6;
		}
		double delta_p;//from database, CFD
		try{
			delta_p = ConfigurationOptions.pressureDrop;
		} catch (Exception e){
			delta_p = -1;
		}
		
		double Vair_total;
		double mair_total;
		
		//in case of DCworms calculation of Vair_total
		double mair_rack = 0;

		for(ComputingResource cr: room.getChildren()){
			ComputeBox1 cb1 = (ComputeBox1)cr;
			for(NodeGroup nodeGroup: cb1.getNodesGroups()){
				for(Device device: nodeGroup.getResourceCharacteristic().getDevices()){
					if(device.getType().equals(StandardResourceType.Outlet)){
						double mair_recs = 0;
						double Vair_recs = 0;
						
						double Vair_recs1 = 0;
						/*try {
							Vair_recs1  = device.getAirThroughputInterface().getAirFlow(device.getAirThroughputInterface().getAirThroughputState());
						} catch (NoSuchFieldException e) {
						}
						*/
						if(device.getAirThroughputInterface().getAirThroughputState().getName().equals(new UserAirThroughputStateName("1").getName())){
							Vair_recs1 = CoolEmAllTestbedMeasurements.SINGLE_FAN_OFF_AIR_FLOW;
						} else {
							Vair_recs1 = CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_AIR_FLOW;
						}
						double Vair_recs2 = 0;
						for(Device device2: nodeGroup.getResourceCharacteristic().getDevices()){
							if(device2.getType().equals(StandardResourceType.Inlet) && device2.getFullName().equals(device.getFullName().replace("Outlet", "Inlet"))){
								/*try {
									Vair_recs2  = device2.getAirThroughputInterface().getAirFlow(device2.getAirThroughputInterface().getAirThroughputState());
								} catch (NoSuchFieldException e) {
								}
								*/
								if(device2.getAirThroughputInterface().getAirThroughputState().getName().equals(new UserAirThroughputStateName("1").getName())){
									Vair_recs2 = CoolEmAllTestbedMeasurements.SINGLE_FAN_OFF_AIR_FLOW;
								} else {
									Vair_recs2 = CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_AIR_FLOW;
								}
								break;
							}
						}	

						Vair_recs  = Math.max(Vair_recs1, Vair_recs2);

						mair_recs = Vair_recs * ro;
						mair_rack  = mair_rack + mair_recs;
					}
				}
			}
		}
		mair_total = mair_rack;
		Vair_total = mair_total / ro;
		
		//in case of CFD calculation of Vair_total
		try{
			Vair_total = ConfigurationOptions.airFlowVolume;
		} catch (Exception e){
			Vair_total = 0;
		}
		mair_total = Vair_total * ro;
		
		//if data not present in DB than don't take Pfans_dc into account
		if(delta_p != -1 && Vair_total != -1)
			Pfans_dc = delta_p * Vair_total / nf;
		
		
		
		/************ Pcooling_device ************/
		
		
		double Pchiller = 0;
		double Pdry_cooler;
		
		/*********** Pchiller ***********/
		double Cp = 1004;//constant
		
		double ncc;//DEBB
		try{
			ncc = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("coolingCoilEfficiency").get(0).getContent()).doubleValue();
		} catch (Exception e){
			ncc = 0.95;
		}
		
		double Qcooling_rated;//DEBB
		try{
			Qcooling_rated = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("coolingCapacityRated").get(0).getContent()).doubleValue();
		} catch (Exception e){
			Qcooling_rated = 3000;
		}
		
		double EERrated;//DEBB
		try{
			EERrated = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("EERRated").get(0).getContent()).doubleValue();
		} catch (Exception e){
			EERrated = 5;
		}
		
		double Tco;//DEBB
		try{
			Tco = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("CWT").get(0).getContent()).doubleValue();
		} catch (Exception e){
			Tco = 30;
		}

		double delta_Th_ex;//DEBB
		try{
			delta_Th_ex = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("deltaThEx").get(0).getContent()).doubleValue();
		} catch (Exception e){
			delta_Th_ex = 10;
		}

		double Tr_out;//from database, CFD
		try{
			Tr_out = ConfigurationOptions.outletRoomAirTempeature;
		} catch (Exception e){
			Tr_out = -1;
		}
		
		double Tr_in;//experiment
		try{
			Tr_in = ConfigurationOptions.inletRoomAirTempeature;
		} catch (Exception e){
			Tr_in = -1;
		}
		
		double Qdc;
		double Qcooling;
		
		double Tev;		
		double CoolT;
		double Qcooling_nom;
		double PLR;

		double delta_T;
		double CoolPR;
		double CoolPRrated;
		double CoolPRT;
		double CoolPRPLR;
		double EER;
		
		
		Qdc = mair_total * Cp * (Tr_out - Tr_in);
		Qcooling = Qdc / ncc;

		Tev = Tr_in - delta_Th_ex;
		double Tevf = 32 + 1.8 * Tev;
		double Tcof = 32 + 1.8 * Tco;
		CoolT = 0.647177 + 0.015888 * Tevf + 0.000103 * Math.pow(Tevf, 2) - 0.004167 * Tcof + 0.000007 * Math.pow(Tcof, 2) - 0.000064 * Tevf * Tcof;
		Qcooling_nom = Qcooling_rated * CoolT;		
		PLR = Qcooling / Qcooling_nom;
		
		delta_T = Tcof - Tevf;
		CoolPRrated = 1 / EERrated;
		CoolPRT = 0.564064 - 0.011463 * Tevf + 0.000112 * Math.pow(Tevf, 2) + 0.008091 * Tcof + 0.000070 * Math.pow(Tcof, 2) - 0.000126 * Tevf * Tcof;
		CoolPRPLR = 0.023918 + 0.889469 * PLR + 0.077931 * Math.pow(PLR, 2) - 0.000264 * delta_T + 0.000007 * Math.pow(delta_T, 2) + 0.000180 * PLR * delta_T;
		CoolPR = CoolPRrated * CoolPRT * CoolPRPLR;
		EER = 1/CoolPR;
		
		Pchiller = Qcooling/EER;

			
		

		/*********** Pdry_cooler ***********/
		double delta_Th_dry_cooler;//DEBB
		try{
			delta_Th_dry_cooler = Double.valueOf(coolingDevice.getResourceCharacteristic().getParameters().get("deltaThDryCooler").get(0).getContent()).doubleValue();
		} catch (Exception e){
			delta_Th_dry_cooler = 10;
		}
		
		double Tamb;//experiment
		try{
			Tamb = ConfigurationOptions.ambientAirTempeature;
		} catch (Exception e){
			Tamb = -1;
		}
		
		Pdry_cooler = 0.02 * Qcooling;
		
		
		//if data not present in DB than don't take Pchiller into account
		if(Vair_total != -1 && Tr_in != -1 && Tr_out != -1) {
			if(Tamb != -1 && Tev - Tamb > 0 && delta_Th_dry_cooler > Tev - Tamb)
				Pcooling_device = Pdry_cooler;
			else
				Pcooling_device = Pchiller;	
		}
		
		

		Pdc = Pload_dc + Pothers + Pfans_dc + Pcooling_device;
		//System.out.println("---");
		//System.out.println("Pdry_cooler: " + Pdry_cooler + " Pchiller: "+ Pchiller);
		//System.out.println("Pdc: " + Pdc + " Pload_dc: "+ Pload_dc + " Pothers: " + Pothers + " Pfans_dc: " + Pfans_dc + " Pcooling_device: " + Pcooling_device);
		
		
		
		
		/**************** HEAT ****************/
		double Qdc2;
		
		double Qload_dc = 0;
		double Qothers = 0;
		double Qfans_dc = 0;

		double delta_2; //DEBB - currently not present;
		delta_2 = 0.4;
		
		for(ComputingResource cr: room.getChildren()){
			ComputeBox1 cb1 = (ComputeBox1)cr;
			double QnodeGroup = 0;
			for(NodeGroup nodeGroup: cb1.getNodesGroups()){
				double Qnodes = 0;
				for(Node node: nodeGroup.getNodes()){
					double Qcpu = 0;
					for(Processor proc: node.getProcessors()){
						Qcpu = Qcpu + proc.getPowerInterface().getRecentPowerUsage().getValue();
					}
					double Qfan = 0;
					for(Device device: node.getParent().getResourceCharacteristic().getDevices()){
						if(device.getFullName().equals(Node_Fan_Mapping.getNode_fan().get(node.getFullName()))){
							if(device.getPowerInterface().getRecentPowerUsage().getValue() == -1){
								Qfan = Qfan + CoolEmAllTestbedMeasurements.SINGLE_FAN_ON_POWER_CONSUMPTION;
							}else
								Qfan = Qfan + device.getPowerInterface().getRecentPowerUsage().getValue();
							break;
						}
					}
					Qnodes = Qnodes + Qcpu + (1 - delta_2) * Qfan;
				}
				QnodeGroup = QnodeGroup + Qnodes;
			}
			Qload_dc  = Qload_dc + QnodeGroup;
		}
		
		Qload_dc = Qload_dc + CoolEmAllTestbedMeasurements.OTHER_DEVICES_POWER_CONSUMPTION;
		
		Qothers = Pothers;
		
		Qfans_dc = (1 - nf) * Pfans_dc;
		
		Qdc2 = Qload_dc + Qothers + Qfans_dc;
		
		double Qdc2v2 = 0;
		
		if(Vair_total != -1 && Tr_in != -1 && Tr_out != -1)
			Qdc2v2 = mair_total * Cp * (Tr_out - Tr_in);

		//System.out.println("Qdc2: " + Qdc2 + " Qdc2v2: " + Qdc2v2 + "diff: " + (Qdc2v2 - Qdc2));

		
		return Pdc;
	}
	

}
